require 'test/unit'
require File.dirname(__FILE__) + '/../lib/staticmatic'
require File.dirname(__FILE__) + '/test_helper'

class HelpersTest < Test::Unit::TestCase
  include StaticMatic::Helpers
  
  def setup
    @staticmatic = StaticMatic::Base.new(File.dirname(__FILE__) + '/sandbox/test_site')
  end

  # Stylesheets tests
  def test_should_generate_stylesheet_links
    set_current_page("/index.html")
    expected_output = %q{<link href="stylesheets/application.css" media="all"}
    assert_match expected_output, stylesheets

    expected_output = %q{<link href="stylesheets/application.css" media="print"}
    assert_match expected_output, stylesheets(:media => :print)

    expected_output = %q{<link href="stylesheets/application.css" media="all"}
    assert_match expected_output, stylesheets(:application)

    expected_output = %q{<link href="stylesheets/application.css" media="print"}
    assert_match expected_output, stylesheets(:application, :media => :print)

    set_current_page("/subdir/index.html")
    expected_output = %q{<link href="../stylesheets/application.css"}
    assert_match expected_output, stylesheets

    set_current_page("/subdir/other/index.html")
    expected_output = %q{<link href="../../stylesheets/application.css"}
    assert_match expected_output, stylesheets
  end
  
  # Link tests
  def test_should_autolink_page
    set_current_page("/index.html")
    expected_output = %q{<a href="test.html">Test</a>}
    assert_match expected_output, link("Test")

    set_current_page("/subdir/index.html")
    expected_output = %q{<a href="../test.html">Test</a>}
    assert_match expected_output, link("Test")

    set_current_page("/subdir/other/index.html")
    expected_output = %q{<a href="../../test.html">Test</a>}
    assert_match expected_output, link("Test")
  end

  def test_should_not_change_url_with_protocol
    expected_output = %q{<a href="mailto:joe@mailinator.com">Email Joe</a>}
    assert_match expected_output, link("Email Joe", "mailto:joe@mailinator.com")

    %w(http https ftp gopher ssh telnet).each do |protocol|
      expected_output = "<a href=\"#{protocol}://www.google.com\">Google</a>"
      assert_match expected_output, link("Google", "#{protocol}://www.google.com")
    end
  end

  def test_should_not_change_url_with_existing_relative_path
    ['/index.html', '/subdir/index.html', '/subdir/other/index.html'].each do |the_current_page|
      set_current_page(the_current_page)

      ['/foo.html', './foo.html', '../other/foo.html', '//www.google.com/search'].each do |the_path| 
        expected_output = "<a href=\"#{the_path}\">Foo</a>"
        assert_match expected_output, link("Foo", the_path)
      end
    end
  end

  def test_should_not_change_anchor_url
    ['/index.html', '/subdir/index.html', '/subdir/other.html'].each do |the_current_page|
      set_current_page(the_current_page)

      expected_output = %q{<a href="#myanchor">Foo</a>}
      assert_match expected_output, link("Foo", "#myanchor")
    end
  end

  def test_should_strip_html_extension_from_page_links
    @staticmatic.configuration.use_extensions_for_page_links = false

    set_current_page("/index.html")
    expected_output = %q{<a href="other">Other</a>}
    assert_match expected_output, link("Other")

    set_current_page("/subdir/index.html")
    expected_output = %q{<a href="../other">Other</a>}
    assert_match expected_output, link("Other")

    set_current_page("/subdir/other/index.html")
    expected_output = %q{<a href="../../other">Other</a>}
    assert_match expected_output, link("Other")

    set_current_page("/index.html")
    expected_output = %q{<a href="/other">Other</a>}
    assert_match expected_output, link("Other", "/other.html")

    expected_output = %q{<a href="/foo/">Foo</a>}
    assert_match expected_output, link("Foo", "/foo/index.html")

    expected_output = %q{<a href="/">Home</a>}
    assert_match expected_output, link("Home", "/")
  end

  def test_should_not_strip_extension_from_non_page_links
    @staticmatic.configuration.use_extensions_for_page_links = false

    set_current_page("/index.html")
    expected_output = %q{<a href="other.zip">Other</a>}
    assert_match expected_output, link("Other", "other.zip")

    set_current_page("/subdir/index.html")
    expected_output = %q{<a href="../other.zip">Other</a>}
    assert_match expected_output, link("Other", "other.zip")

    set_current_page("/subdir/other/index.html")
    expected_output = %q{<a href="../../other.zip">Other</a>}
    assert_match expected_output, link("Other", "other.zip")

    set_current_page("/index.html")
    expected_output = %q{<a href="/other.zip">Other</a>}
    assert_match expected_output, link("Other", "/other.zip")

    expected_output = %q{<a href="http://www.google.com/index.html">Google</a>}
    assert_match expected_output, link("Google", "http://www.google.com/index.html")
  end

  def test_should_prefix_relative_path_while_generating_file_name
    ['/other.html', '/subdir/index.html', '/subdir/other/filename.html'].each do |the_current_page|
      set_current_page(the_current_page)

      ['Other', 'Other File', 'Other / File', 'Other /File/'].each do |link_name|
        ['../', './', '/', '/foo/','../bar/'].each do |relative_path| 
          expected_output = %Q{<a href="#{relative_path}#{urlify(link_name)}.html">#{link_name}</a>}
          assert_match expected_output, link("#{relative_path}#{link_name}")
        end
      end
    end
  end

  def test_should_not_prefix_relative_path_while_generating_file_name
    set_current_page("/other.html")

    expected_output = %q{<a href="testlink.html">Test/Link</a>}
    assert_match expected_output, link('Test/Link')

    set_current_page("/subdir/index.html")
    expected_output = %q{<a href="../testlink.html">Test/Link</a>}
    assert_match expected_output, link('Test/Link')

    set_current_page("/subdir/other/filename.html")
    expected_output = %q{<a href="../../testlink.html">Test/Link</a>}
    assert_match expected_output, link('Test/Link')
  end

  # Tag tests
  def test_should_generate_tag_with_block
    expected_output = %q{<a href="/test.html" title="My Test Link">Test</a>}
    assert_match expected_output, tag(:a, :href => "/test.html", :title => 'My Test Link') { "Test" }
  end
  
  def test_should_generate_tag
    expected_output = %q{<br/>}
    assert_match expected_output, tag(:br)
  end
  
  # Urlify tests
  def test_should_urlify_string
    assert_equal "stephens_haml_and_sass_project", urlify("Stephen's Haml & Sass Project")
    assert_equal "testlink", urlify("Test/Link")
  end
  
  # Text_field tests
  def test_should_generate_input
    expected_output = %q{<input type="text" id="test" value="blah" name="test"/>}
    assert_tags_match expected_output, text_field("test", "blah")
  end
  
  # Text_area tests
  def test_should_generate_text_area
    expected_output = '<textarea id="hello" name="hello">world</textarea>'
    assert_equal expected_output, text_area('hello', 'world')
  end
  
  # Element id test
  def test_id_should_not_be_overwritten
    expected_output = '<textarea id="goodbye" name="hello">world</textarea>'
    assert_equal expected_output, text_area('hello', 'world', :id => 'goodbye')
  end

  # Javascripts tests
  def test_should_generate_js_links
    set_current_page('/index.html')
    expected_output = %q{src="javascripts/test.js"}
    assert_match expected_output, javascripts('test')

    expected_output = %q{src="javascripts/test.js"}
    assert_match expected_output, javascripts(:test)

    set_current_page('/subdir/index.html')
    expected_output = %q{src="../javascripts/test.js"}
    assert_match expected_output, javascripts('test')

    set_current_page('/subdir/other/index.html')
    expected_output = %q{src="../../javascripts/test.js"}
    assert_match expected_output, javascripts('test')
  end

  def test_should_not_change_js_link_with_existing_relative_url
    ['/index.html', '/subdir/index.html', '/subdir/other/index.html'].each do |the_current_page|
      set_current_page(the_current_page)

      ['/my_javascript.js', './my_javascript.js', '../other/my_javascript.js'].each do |the_path|
        expected_output = "src=\"#{the_path}\""
        assert_match expected_output, javascripts(the_path)
      end
    end
  end

  def test_should_generate_absolute_js_link
    ['/index.html', '/subdir/index.html', '/subdir/other/foo.html'].each do |the_current_page|
      set_current_page(the_current_page)

      ['http://staticmatic.rubyforge.org/javascripts/test.js', 'https://staticmatic.rubyforge.org/javascripts/test.js'].each do |the_path|
        expected_output = "src=\"#{the_path}\""
        assert_match expected_output, javascripts(the_path)
      end
   end
  end
  
  # Partials tests
  def test_should_include_partial_template
    @staticmatic.instance_eval { def current_file() "/index.haml"; end }
    set_current_page("/index.html")
    expected_output = "My Menu"
    assert_match expected_output, partial("menu")
  end
  
  # Image tests
  def test_should_generate_basic_img_tag
    set_current_page("/index.html")
    expected_output = %q{<img src="images/test.gif" alt="Test"/>}
    assert_tags_match expected_output, img('test.gif')

    set_current_page("/subdir/index.html")
    expected_output = %q{<img src="../images/test.gif" alt="Test"/>}
    assert_tags_match expected_output, img('test.gif')

    set_current_page("/subdir/other/index.html")
    expected_output = %q{<img src="../../images/test.gif" alt="Test"/>}
    assert_tags_match expected_output, img('test.gif')
  end

  def test_should_generate_img_tag_with_options
    set_current_page("/index.html")
    expected_output = %q{<img src="images/test.gif" alt="Testing the alt" class="testClass"/>}
    assert_tags_match expected_output, img('test.gif', {:alt => 'Testing the alt', :class => 'testClass'})
  end
  
  def test_should_generate_img_tag_with_sub_directory
    set_current_page("/index.html")
    expected_output = %q{<img src="images/about_us/test.gif" alt="Test"/>}
    assert_tags_match expected_output, img('about_us/test.gif')
  end
  
  def test_should_generate_img_tag_with_src_left_as_is
    ['/index.html', '/subdir/index.html', '/subdir/other/index.html'].each do |the_current_page|
      set_current_page(the_current_page)

      ['/images/test_image.gif', './test_image.gif', '../test_image.gif'].each do |the_path|
        expected_output = "<img src=\"#{the_path}\" alt=\"Test image\"/>"
        assert_tags_match expected_output, img(the_path)
      end
    end
  end
  
  def test_should_generate_absolute_img_tag
    ['/index.html', '/subdir/foo.html', '/subdir/other/index.html'].each do |the_current_page|
      set_current_page(the_current_page)

      ['http://staticmatic.rubyforge.org/images/bycurve21.gif', 'https://staticmatic.rubyforge.org/images/bycurve21.gif'].each do |the_path| 
        expected_output = "<img src=\"#{the_path}\" alt=\"Bycurve21\"/>"
        assert_tags_match expected_output, img(the_path)
      end
    end
  end
  
  private

  def set_current_page(page)
    @staticmatic.instance_variable_set("@current_page", page)
  end

end
